# Mimi Internals

## Overview

This document provides a forward-looking description of the internal architecture of Mimi. That is, we describe here where Mimi is going, not where it has been.

Several improvements are planned, relative to the prototype (Julia) version of Mimi. These include:

1. Simplification of internal structure by overloading the "." operator
2. Simplification of the @defcomp macro to improve maintainability
3. Support for [Monte Carlo](montecarlo.html) simulation
4. A simple graphical user interface for viewing model results as plots
5. Renaming of internal structures and types for clarity and consistency
6. other stuff?

## Renaming of internal structure / types
We are retaining the distinction between (i) descriptive objects that define the structure (and default values) for models and their components, and (ii) model run-time objects that represent actual model and component instances (i.e., numerous instances of which might be running on a cluster)

* Descriptive object names end with ```Def```, as in ```ModelDef``` and ```ComponentDef```.
* Run-time objects are the same names without the ```Def```, i.e., ```Model``` and ```Component```.

### Name changes
* ```Model``` becomes ```ModelDef```
* ```ModelInstance``` becomes ```Model```
* ```ComponentInstanceInfo``` becomes ```ComponentDef```
* ```ComponentState```, which was a super-type of a custom type generated by @defcomp for each component has been eliminated. In its place, a new ```Component``` type is used to store run-time info for individual components.
  * Instead of generating a custom type for each component, dispatch to the component's ```run_timestep``` method is handled using the construct ```Var{:xxx}```, where ```:xxx``` is the name of a component.
  * ```run_timestep(Val{compname}, v, p, d, t)```
  * ```run_timestep(s::Symbol, args...) = run_timestep(Val(s), args...)```

## Dot overloading
TBD. (David to implement initial version.)

## Graphical User Interface
* ````explorer```` function produces a simple graphical user interface that can be used to view the variables and
parameters of a model instance.

